<?php
/**
 * Plugin Name:         RS Updater
 * Description:         Allow automatic updating of plugins from plugins.seindal.dk
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-updater/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-updater
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             1.0.2
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action( 'plugins_loaded', function() {
    class PluginUpdater extends PluginBase {
        protected $plugin_file = __FILE__;

        // The backend should run the wordpress-plugin-site plugin
        protected $repository = 'https://plugins.seindal.dk';

        /*
         * This is needed in case the repository appears on a local
         * network from the point of view of the client.
         */

        function do_http_request_host_is_external_filter( $value, $host, $url ) {
            if ( false === $value ) {
                $repository_host = wp_parse_url( $this->get_repository(),  PHP_URL_HOST );
                $value = ( $repository_host === $host );
            }
            return $value;
        }

        /*
         * Save list of updatable plugins
         */

        function do_site_transient_update_plugins_filter( $value, $transient ) {
            global $wp_version;

            if ( empty( $value) )
                return $value;

            $rsplugins = $this->get_available_plugins();
            if ( empty( $rsplugins ) )
                return $value;

            $installed = get_plugins();
            foreach ( $rsplugins as $slug => $versions ) {
                $plugin = "$slug/$slug.php";

                if ( ! array_key_exists( $plugin, $installed ) )
                    continue;

                $data = $versions['stable'] ?? $versions['latest'];

                $installed_version = $installed[$plugin]['Version'];

                if ( version_compare( $installed_version, $data['new_version'], '<' ) )
                    $value->response[ $plugin ] = (object)$data;
                else
                    $value->no_update[ $plugin ] = (object)$data;
            }

            return $value;
        }

        /*
         * Provide info for the "View details" link on the Plugin list
         */

        function do_plugins_api_filter( $def, $action, $args ) {
            if ( 'plugin_information' === $action ) {
                $slug = $args->slug;
                if ( empty( $slug ) )
                    return $def;

                $plugins = $this->get_available_plugins();
                if ( empty( $plugins ) )
                    return $def;

                if ( empty( $plugins[$slug] ) )
                    return $def;

                $info = $this->get_plugin_info( $slug );
                if ( $info )
                    return (object) $info;
            }

            return $def;
        }



        /*
         * Add our search results rather than overrule WP plugin registry
         * This messes a bit with paging of results
         */

        function do_plugins_api_result_filter( $res, $action, $args ) {
            if ( 'query_plugins' === $action ) {
                if ( empty( $args->search ) || $args->page !== 1 )
                    return $res;

                $rsplugins = $this->get_available_plugins();
                if ( empty( $rsplugins ) )
                    return $res;

                $found = [];
                foreach ( $rsplugins as $slug => $versions ) {
                    if ( str_contains( $slug, $args->search ) )
                        $found[] = $this->get_plugin_info( $slug );
                }

                if ( $found ) {
                    $res->plugins = array_merge( $found, $res->plugins );
                    $res->info['results'] += count( $found );
                }
            }

            return $res;
        }

        /*
         * Get the repository url
         */

        protected function get_repository() {
            return $this->get_option( 'repository', $this->repository );
        }

        /*
         * Get the plugin index with micro-caching
         */

        protected $plugin_index_transient = 'rs_updater_plugin_index';

        protected function get_available_plugins() {
            return $this->get_json_url_with_micro_caching( $this->get_repository() . '/plugin-index/', $this->plugin_index_transient );
        }

        /*
         * Get the plugin info with micro-caching
         */

        protected $plugin_info_transient = 'rs_updater_plugin_info_';

        protected function get_plugin_info( $slug ) {
            $transient = $this->plugin_info_transient . $slug;
            return $this->get_json_url_with_micro_caching( $this->get_repository() . "/plugins/{$slug}/info/", $transient );
        }

        /*
         * Do the micro-caching part
         */

        function get_json_url_with_micro_caching( $url, $transient ) {
            $data = get_transient( $transient );

            if ( empty( $data ) ) {
                $json = @file_get_contents( $url );

                if ( ! empty( $json ) ) {
                    try {
                        $data = json_decode( $json, true );
                        set_transient( $transient, $data, 5 * MINUTE_IN_SECONDS );
                    } catch ( ValueException ) {
                        $data = NULL;
                    }
                }
            }

            return $data;

        }

        function get_json_url( $url ) {
            $json = @file_get_contents( $url );
            $data = NULL;

            if ( ! empty( $json ) ) {
                try {
                    $data = json_decode( $json, true );
                } catch ( ValueException ) {
                    $data = NULL;
                }
            }

            return $data;
        }


        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        use PluginBaseSettings;

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'RS Updater', 'rs-updater' ),
                __( 'RS Updater', 'rs-updater' )
            );
        }

        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_registry',
                __( 'Plugin repository', 'rs-updater' ),
                __( 'Please enter the settings relative to the plugin repository.', 'rs-updater' )
            );

            $this->settings_add_field(
                'repository', $section,
                __( 'Repository URL', 'rs-updater' ),
                'settings_field_input_html',
                [
                    'default' => $this->repository,
                    'size' => 40,
                ]
            );
        }

        function settings_sanitize_option( $settings ) {
            if ( !empty( $settings['repository'] ) && wp_http_validate_url( $settings['repository'] ) ) {
                $plugins = $this->get_json_url( $settings['repository'] . '/plugin-index/' );

                if ( empty( $plugins ) )
                    add_settings_error(
                        $this->option_name, sanitize_key( 'repository' ),
                        __( 'The repository URL is invalid.', 'rs-updater' )
                    );
                else
                    add_settings_error(
                        $this->option_name, sanitize_key( 'repository' ),
                        sprintf( __( 'Found %d plugins in the repository', 'rs-updater' ), count( $plugins ) ),
                        'success'
                    );
            }

            return $settings;
        }
    }

    PluginUpdater::instance();
} );
